package x36dsv.sebekji1.cvi2.client;

import java.io.FileNotFoundException;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import x36dsv.sebekji1.cvi2.server.CsvReader;
import x36dsv.sebekji1.cvi2.server.DBExistException;
import x36dsv.sebekji1.cvi2.server.DBNotFoundException;
import x36dsv.sebekji1.cvi2.server.DuplicateKeyException;
import x36dsv.sebekji1.cvi2.server.KeyNotFoundException;
import x36dsv.sebekji1.cvi2.compute.*;

public class Client {
/*
	private static final String K_listDB = "listdb";
	private static final String K_createDB = "createdb";
	private static final String K_insert = "insert";
	private static final String K_update = "update";
	private static final String K_get = "get";
	private static final String K_getA = "geta";
	private static final String K_flush = "flush";
*/
	public static void main(String[] args) {
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
		}

		String host;
		int PORT;
		String conf_file;

		if (args.length != 3) {
			// <IP_adresa> <cislo_portu> <konf_soubor>

			host = "localhost"; //host = "127.0.0.1";
			PORT = 7777; 
			conf_file = "./data/conf.csv"; // konfig. soubor
			// System.exit(0);
		} else {
			host = args[0]; 
			PORT = Integer.valueOf(args[1]); 
			conf_file = args[2]; 
		}

		try {
			// vyhledani vzdaleneho objektu
			ServerInterface serveri;
			String name = "muj_server";
			Registry registry = LocateRegistry.getRegistry(host, PORT);

			 serveri = (ServerInterface) registry.lookup(name);
			//serveri = (ServerInterface) Naming.lookup("//"+host+":"+PORT+"/"+name);
			 CsvReader reader = null;
			 try{
				 reader = new CsvReader(conf_file, ';');
			 }catch (Exception e) {
					System.out.println("Cannot find a config file.");
					System.exit(1);
			 }
			while (reader.readRecord()) {
				String command = reader.get(0);
				String dbname = "";
				if (reader.getColumnCount() > 1) {
					dbname = reader.get(1);
				}
				switch (command) {
					case "listdb":
						try {
							System.out.println(">> listdb");

							String[] dbseznam = serveri.listDB();

							System.out.print("<< Databases: ");
							for (int i = 0; i < dbseznam.length; i++) {
								System.out.print(dbseznam[i] + ", ");
							}
							System.out.println("");
						} catch (Exception e) {
							System.out.println("Cannot connect to server.");
						}
						break;
					case "flush":
						try {
							System.out.println("<< Database - Writing changes to disk.");
							serveri.flush();
						} catch (Exception e) {
							System.out.println("Cannot connect to server.");
						}
						break;
					case "createdb":
						try {
							System.out.println(">> createdb " + dbname);
							if (serveri.createDB(dbname)) {
								System.out.println("<< Database \"" + dbname + "\" created.");
							} else {
								System.out.println("<< Database \"" + dbname + "\" already exist.");
							}
						} catch (RemoteException e) {
							if (e != null && e.getCause() != null) {
								RemoteException re = new RemoteException();
								if (e.getCause() instanceof DBExistException) {
									re = (DBExistException) e.getCause();
								}
								System.out.println("<< " + re.getMessage());
							} else {
								System.out.println("Neznm chyba");
							}
						}
						break;
					case "insert":
						try {
							System.out.println(">> Inserting into database \"" + dbname + "\" record [\"" + reader.get(2) + "\";\"" + reader.get(3) + "\"]");
							if (serveri.insert(dbname, Integer.valueOf(reader.get(2)), reader.get(3)) == 0) {
								System.out.println("<< DB \"" + dbname + "\" - record inserted");
							} else {
								System.out.println("<< insert se nezdail");
							}
						} catch (RemoteException e) {
							if (e != null && e.getCause() != null) {
								RemoteException re = new RemoteException();
								if (e.getCause() instanceof DBNotFoundException) {
									re = (DBNotFoundException) e.getCause();
								}
								if (e.getCause() instanceof DuplicateKeyException) {
									re = (DuplicateKeyException) e.getCause();
								}
								System.out.println("<< " + re.getMessage());
							} else {
								System.out.println("Neznm chyba");
							}
						}
						break;
					case "update":
						try {
							System.out.println(">> Updating record \"" + reader.get(2) + "\" in database \"" + dbname + "\" with \"" + reader.get(3) + "\"");
							if (serveri.update(dbname, Integer.valueOf(reader.get(2)), reader.get(3)) == 0) {
								System.out.println("<< DB \"" + dbname + "\" - record updated");
							} else {
								System.out.println("<< update se nezdail");
							}
						} catch (RemoteException e) {
							if (e != null && e.getCause() != null) {
								RemoteException reex = new RemoteException();
								if (e.getCause() instanceof DBNotFoundException) {
									reex = (DBNotFoundException) e.getCause();
								}
								if (e.getCause() instanceof KeyNotFoundException) {
									reex = (KeyNotFoundException) e.getCause();
								}
								System.out.println("<< " + reex.getMessage());
							} else {
								System.out.println("Neznm chyba");
							}
						}

						break;
					case "get":
						try {
							System.out.println(">> Getting record with key \"" + reader.get(2) + "\" from database \"" + dbname + "\"");
							DBRecord rec = serveri.get(dbname, Integer.valueOf(reader.get(2)));
							if (rec != null) {
								System.out.println("<< DB \"" + dbname + "\" - " + rec.toString());
							} else {
								System.out.println("<< get se nezdail");
							}
						} catch (RemoteException e) {
							if (e != null && e.getCause() != null) {
								RemoteException reex = new RemoteException();

								if (e.getCause() instanceof DBNotFoundException) {
									reex = (DBNotFoundException) e.getCause();
								}

								if (e.getCause() instanceof KeyNotFoundException) {
									reex = (KeyNotFoundException) e.getCause();
								}
								System.out.println("<< " + reex.getMessage());
							} else {
								System.out.println("Neznm chyba");
							}
						}
						break;
					case "geta":
						try {
							int countTemp = reader.getColumnCount() - 2; // 3.ti sloupec
							Integer[] keys = new Integer[countTemp];
							String keysString = "";
							int count = reader.getColumnCount()-2;
							for (int i = 0; i < count; i++) {
								keys[i] = Integer.valueOf(reader.get(i+2));
								keysString += keys[i] + ", ";
							}
							System.out.println(">> Getting records with keys \"" + keysString + "\" from database \"" + dbname + "\"");

							DBRecord[] records = serveri.getA(dbname, keys);

							if (records != null) {
								System.out.println("<< Records from database \"" + dbname + "\": ");
								for (int i = 0; i < records.length; i++) {
									System.out.println("\t" + keys[i] + ": " + records[i].toString());
								}

							} else {
								System.out.println("<< get se nezdail");
							}
						} catch (RemoteException e) {

							if (e != null && e.getCause() != null) {

								RemoteException re = new RemoteException();
								if (e.getCause() instanceof DBNotFoundException) {
									re = (DBNotFoundException) e.getCause();
								}
								if (e.getCause() instanceof KeyNotFoundException) {
									re = (KeyNotFoundException) e.getCause();
								}
								System.out.println("<< " + re.getMessage());
							} else {
								System.out.println("Neznm chyba");
							}
						}
						break;
					default:
						System.out.println(">> ERROR  command \"" + command + "\" not implemented");
						break;
				}
			}
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		} catch (Exception e) {
			System.err.println("Data exception: " + e.getMessage());
		}
	}
}
